# Options for Addons

> This document is generated by *dev/docgen/make_options_for_addons.js* from source files.
> If you want to update this doc, edit corresponded source code.
>
> Based on HyperMD v0.0.1 [( with uncommited changes, git a6530e01 )](https://github.com/laobubu/HyperMD/tree/a6530e01)

Once a HyperMD addons are loaded, you may configure them via editor options.

# ✨ Usage

## While Initializing

Both CodeMirror and HyperMD provide a method `fromTextArea`

```js
var cm = HyperMD.fromTextArea(myTextarea, {
  /* editor options here */

  // example 1
  hmdClick: {
    enabled: true,
    handler: onClickMarkdown,
  },

  // example 2. hmdHideToken also accepts boolean
  hmdHideToken: false,
})
```

## During Runtime

Use `setOption` method of the editor

```js
// example 3, use setOption method
cm.setOption("hmdClick", {
  enabled: false,
})
```

> :warning: **You shall use `setOption`**
>
> Sometimes, directly accessing _addon instance_'s property like `cm.hmd.Click.enabled = false;`
> **might** work for some addons / options, but it's **not guranteed** (depends on the addon).
> Take the non-standard approach at your own risk.



# 📕 Options

| Name | Addon | Addon Description |
| ---- | ---- | ---- |
| hmdInsertFile | [addon/insert-file](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts) | Insert images or files into Editor by pasting (Ctrl+V) or Drag'n'Drop |
| hmdReadLink | [addon/read-link](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/read-link.ts) | Fetch footnote content, Resolve relative URLs |
| hmdHover | [addon/hover](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts) | When mouse hovers on a link or footnote ref, shows related footnote |
| hmdClick | [addon/click](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts) | Click to open links / jump to footnotes / toggle TODOs, and more. |
| hmdPaste | [addon/paste](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts) | Convert content to Markdown before pasting |
| hmdFold | [addon/fold](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold.ts) | Turn Markdown markers into real images, link icons etc. Support custom folders. |
| hmdFoldCode | [addon/fold-code](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts) | Turn code blocks into flow charts / playground sandboxes etc. |
| hmdFoldMath | [addon/fold-math](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts) | Fold and Render TeX formula expressions. Works with *fold* addon. |
| hmdFoldEmoji | [addon/fold-emoji](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts) | Fold and render emoji :smile: |
| hmdFoldHTML | [addon/fold-html](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts) | Fold and render embedded HTML snippets |
| hmdTableAlign | [addon/table-align](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/table-align.ts) | Align Table Columns |
| hmdModeLoader | [addon/mode-loader](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts) | Load code highlighting modes (aka. profiles) automatically |
| hmdHideToken | [addon/hide-token](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hide-token.ts) | Auto show/hide markdown tokens like `##` or `*` |
| hmdCursorDebounce | [addon/cursor-debounce](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/cursor-debounce.ts) | A workaround for cheap and unstable mouses. |





## hmdInsertFile

📦 **Provided by [addon/insert-file](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts)** : Insert images or files into Editor by pasting (Ctrl+V) or Drag'n'Drop

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ [FileHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L37)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L87) for [InsertFile](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L161).
> 
> You may also provide a `false` to disable it; a `true` to enable it with `defaultOption.fileHandler`
> ( Note: you shall load a related PowerPack, or manually, to populate `defaultOption.fileHandler` )
> 
> Or provide a [FileHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L37) (overwrite the default one), meanwhile, byDrop & byPaste will set to `true`

| Name | Type | Description |
| ---- | ---- | ---- |
| byPaste | `boolean` | enable uploading from clipboard |
| byDrop | `boolean` | enable drag n drop uploading |
| fileHandler |   | *(See below)* |

### fileHandler

🎨 **Type** : `(files: FileList, action: HandlerAction) => boolean`

> handler function.
> 
> returns `true` means files are accepted and uploading, and HyperMD will put a placeholder there.
> Then [FileHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L37) may use `action` object to change the placeholder and finish uploading.
> 
> 👉 ***see*** [FileHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L37)
> 👉 ***see*** [HandlerAction](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts#L12)





## hmdReadLink

📦 **Provided by [addon/read-link](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/read-link.ts)** : Fetch footnote content, Resolve relative URLs

🎨 **Type** : `Partial<Options>` _or_ `string`

> If not empty, this will affect `editor.hmdResolveURL()` if the URL of result is relative.
> 
> Also affects other addons, eg. opening links, showing images...

| Name | Type | Description |
| ---- | ---- | ---- |
| baseURI |   | *(See below)* |

### baseURI

🎨 **Type** : `string`

> BaseURI (without filename) used to resolve relative URLs
> If not empty, this will affect `editor.hmdResolveURL("./relative/url")`
> 
> 👉 ***example*** `"https://laobubu.net/HyperMD/docs/zh-CN/"`





## hmdHover

📦 **Provided by [addon/hover](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts)** : When mouse hovers on a link or footnote ref, shows related footnote

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ [Convertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L15)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L32) for [Hover](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L105).
> 
> You may also provide a `false` to disable it;
> a `true` to enable it with defaultOption (except `enabled`);
> or a [Convertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L15) to decide the content of tooltip.
> 
> 👉 ***see*** [Convertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L15)

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [Hover](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L105) features or not. |
| xOffset | `number` |  |
| convertor |   | *(See below)* |

### convertor

🎨 **Type** : `(footnote: string, markdown: string) => string`

> function to decide the tooltip's content.
> 
> First parameter is the name of footnote (without square brackets),
> and the second is footnote Markdown content (might be `null`, if not found).
> This function shall returns HTML string or `null`.
> 
> 👉 ***see*** [Convertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L15)
> 👉 ***see*** [defaultConvertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts#L26)





## hmdClick

📦 **Provided by [addon/click](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts)** : Click to open links / jump to footnotes / toggle TODOs, and more.

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ [ClickHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts#L35)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts#L138) for [Click](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts#L199).
> 
> You may also provide a `false` to disable it; a `true` to enable it with default behavior;
> or a callback which may return `false` to supress HyperMD default behavoir.

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [Click](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts#L199) features or not. |
| handler |   | *(See below)* |

### handler

🎨 **Type** : `(info: ClickInfo, cm: cm_t) => (false | void)`

> A callback when user clicked on something. May return `false` to supress HyperMD default behavoir.
> 
> 👉 ***see*** [ClickHandler](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/click.ts#L35)





## hmdPaste

📦 **Provided by [addon/paste](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts)** : Convert content to Markdown before pasting

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ [PasteConvertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts#L11)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts#L15) for [Paste](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts#L74).
> 
> You may set a `PasteConvertor` function which accepts HTML, returning markdown text. Or set to `null` to disable this feature
> 
> 👉 ***see*** [PasteConvertor](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts#L11)

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [Paste](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts#L74) feature or not. |
| convertor | `(html: string) => string | void` | a function which accepts HTML, returning markdown text. |





## hmdFold

📦 **Provided by [addon/fold](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold.ts)** : Turn Markdown markers into real images, link icons etc. Support custom folders.

🎨 **Type** : [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold.ts#L248) _or_ `boolean`

> Enable/disable registered folders, for current editor instance.
> 
> `hmdFold` accepts:
> 
> 1. `true` -- only enable suggested folders
> 2. `false` -- disable all kinds of folders
> 3. `{ [FolderType]: boolean }` -- enable / disable folders
>     - Note: registered but not configured folder kinds will be disabled
> 
> 👉 ***example*** `{ image: true, link: true, math: true, html: false }`





## hmdFoldCode

📦 **Provided by [addon/fold-code](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts)** : Turn code blocks into flow charts / playground sandboxes etc.

🎨 **Type** : [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts#L152) _or_ `boolean`

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts#L152) for [FoldCode](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts#L218).
> 
> The flow charts / playground sandboxes from codeblocks, are rendered by [CodeRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts#L60) s.
> 
> By default, `suggested` renderers will be enabled.
> You can still choosed to enable/disable some of them via `hmdFoldCode` the editor option.
> 
> **NOTE**: make sure `hmdFold.code` is `true`, otherwise, this option will not work.
> 
> `hmdFoldCode` accepts 3 forms:
> 
> 1. `true` -- only enable suggested renderers
> 2. `false` -- disable all renderers
> 3. `{ [RendererType]: boolean }` -- enable / disable [CodeRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts#L60)
>     - Note: registered but not configured kinds will be disabled





## hmdFoldMath

📦 **Provided by [addon/fold-math](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts)** : Fold and Render TeX formula expressions. Works with *fold* addon.

🎨 **Type** : `Partial<Options>` _or_ (typeof [MathRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L23))

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L198) for [FoldMath](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L268).
> 
> **NOTE**: to switch this feature off, please modify `hmdFold.math` instead.
> 
> You may also provide a [MathRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L23) class constructor
> 
> 👉 ***see*** [MathRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L23)

| Name | Type | Description |
| ---- | ---- | ---- |
| renderer |   | *(See below)* |
| onPreview | `(expr: string) => void` | a callback whenever you shall show/update a math preview |
| onPreviewEnd | `() => void` | a callback whenever you shall hide the preview box |

### renderer

🎨 **Type** : `typeof MathRenderer`

> custom renderer
> 
> 👉 ***see*** [MathRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L23)
> 👉 ***see*** [DumbRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts#L162)





## hmdFoldEmoji

📦 **Provided by [addon/fold-emoji](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts)** : Fold and render emoji :smile:

🎨 **Type** : `Partial<Options>`

> **NOTE**: to stop folding emojis, please modify `hmdFold.emoji` instead.
> 
> `hmdFoldEmoji` is options for EmojiFolder, which also accepts
> 
> - **[EmojiRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts#L13)** function
> - **string**: name of a emoji renderer (see emojiRenderer)

| Name | Type | Description |
| ---- | ---- | ---- |
| myEmoji |   | *(See below)* |
| emojiRenderer |   | *(See below)* |
| emojiChecker | `(text: string) => boolean` | Check if a emoji text , eg. `:smile:` , is valid |

### myEmoji

🎨 **Type** : `{ [name: string]: EmojiRenderer }`

> you may add your custom emojis, which have higher priority than standard emojis
> 
> 👉 ***example*** `{ ":doge:": a_function_that_creates_doge_img_element }`

### emojiRenderer

🎨 **Type** : `(text: string) => HTMLElement`

> Tired of plain text? You may provide a [EmojiRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts#L13) function,
> which generates a HTML Element (eg. emoji image from twemoji or emojione)
> for standard emojis.
> 
> Note that if [EmojiRenderer](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts#L13) returns null, the folding process will be aborted.





## hmdFoldHTML

📦 **Provided by [addon/fold-html](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts)** : Fold and render embedded HTML snippets

🎨 **Type** : `Partial<Options>` _or_ [CheckerFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts#L14)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts#L161) for HTMLFolder
> 
> **NOTE**: to switch this feature off, please modify `hmdFold.html` instead.
> 
> You may provide a [CheckerFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts#L14) to check if a HTML is safe to render.

| Name | Type | Description |
| ---- | ---- | ---- |
| checker | `(html: string, pos: Position, cm: cm_t) => boolean` | Before folding HTML, check it to avoid XSS attack! Returns `true` if safe. |
| renderer | `(html: string, pos: Position, cm: cm_t) => HTMLElement` | A [RendererFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-html.ts#L30) accepts HTML string (which has only one root node), renders it and returns the root element node |
| stubText | `string` | There MUST be a stub icon after rendered HTML. You may decide its content. |
| isolatedTagName | `RegExp` | If the rendered element's tagName matches this, user can NOT break it by clicking it |





## hmdTableAlign

📦 **Provided by [addon/table-align](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/table-align.ts)** : Align Table Columns

🎨 **Type** : `Partial<Options>` _or_ `boolean`

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/table-align.ts#L14) for [TableAlign](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/table-align.ts#L67).
> 
> You may also provide a boolean to toggle it.

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [TableAlign](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/table-align.ts#L67) |





## hmdModeLoader

📦 **Provided by [addon/mode-loader](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts)** : Load code highlighting modes (aka. profiles) automatically

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ `string` _or_ [LoaderFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts#L14)

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts#L17) for [ModeLoader](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts#L81).
> 
> You may also provide:
> - boolean: `true` will use suggested source
> - `string` or `LoaderFunc` as the new source
> 
> 👉 ***see*** [LoaderFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts#L14)

| Name | Type | Description |
| ---- | ---- | ---- |
| source |   | *(See below)* |

### source

🎨 **Type** : `string` _or_ [LoaderFunc](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/mode-loader.ts#L14)

> providing a source of codemirror modes
> 
> - (a `LoaderFunc` function)
> - `"http://cdn.xxxxx.com/codemirror/v4.xx/"`
> - `"./node_modules/codemirror/"`            <- relative to webpage's URL
> - `"~codemirror/"`                          <- for requirejs





## hmdHideToken

📦 **Provided by [addon/hide-token](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hide-token.ts)** : Auto show/hide markdown tokens like `##` or `*`

🎨 **Type** : `Partial<Options>` _or_ `boolean` _or_ `string` _or_ `string[]`

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hide-token.ts#L16) for [HideToken](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hide-token.ts#L87).
> 
> You may also provide a `false` to disable it; a `true` to enable it with defaultOption (except `enabled`);
> or token types (as string array, or just a string with "|" as separator inside)

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [HideToken](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hide-token.ts#L87) features or not. |
| line | `boolean` | Add `hmd-inactive-line` style to inactive lines or not |
| tokenTypes | `string[]` | 👉 ***internal*** reserved yet |





## hmdCursorDebounce

📦 **Provided by [addon/cursor-debounce](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/cursor-debounce.ts)** : A workaround for cheap and unstable mouses.

🎨 **Type** : `Partial<Options>` _or_ `boolean`

> [Options](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/cursor-debounce.ts#L21) for [CursorDebounce](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/cursor-debounce.ts#L72).
> 
> You may also provide a `false` to disable it; a `true` to enable it

| Name | Type | Description |
| ---- | ---- | ---- |
| enabled | `boolean` | Enable [CursorDebounce](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/cursor-debounce.ts#L72) features or not. |