# PowerPacks

> This document is generated by *dev/docgen/make_powerpacks.js* from source files.
> If you want to update this doc, edit corresponded source code.
>
> Based on HyperMD v0.0.1 [( with uncommited changes, git a6530e01 )](https://github.com/laobubu/HyperMD/tree/a6530e01)

👈 *The built-in PowerPacks list is on the left*

**PowerPacks** are optional modules that integrate 3rd-party libs and services to HyperMD.

Note that PowerPacks will `require` related third-party libraries,
and update some HyperMD components' default options.

✨ **Usage (for bundler and module-loader users)**:

Before initializing a editor, import `'hypermd/powerpack/some-powerpack-name'` the module.

✨ **Usage (in plain browser env)**:

When HyperMD main modules and related third-party libs are loaded,
`<script src="PATH_TO_HyperMD/powerpack/some-powerpack-name.js"></script>`

Note that in plain browser env, PowerPack-exported things are stored in
`window.HyperMD_PowerPack['some-powerpack-name']`

------



## fold-code-with-flowchart

📦 **Third-Party Libs**:   [codemirror](https://www.npmjs.com/package/codemirror)  [flowchart.js](https://www.npmjs.com/package/flowchart.js)

🚀 **Related Components**:   [addon/fold-code](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts)  [addon/fold](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold.ts)

🚢 **Provides**:

* [**FlowchartRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-code-with-flowchart.ts#L36)   Type: CodeRenderer

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-code"

This module provides `FlowchartRenderer` for FoldCode addon
so that you can render flowchart with [flowchart.js](http://flowchart.js.org/)

By default the renderer is enabled. You may disable it by setting `hmdFoldCode.flowchart` to `false`

**Example**: https://laobubu.net/HyperMD/docs/examples/flowchart.html

:hint: to change flowchart.js configuration, use `editor.setOption("flowchart", { /* YOUR_CONFiG */ })`

:warning: **flowchart.js bug**

If you are using RequireJS, add `"Raphael": "raphael"` to its `paths` option like this:

```js
requirejs.config({
  // ...
  paths: {
    "Raphael": "raphael", // adapt flowchart.js bug
  },
  //...
})
```



## fold-code-with-mermaid

📦 **Third-Party Libs**:   [codemirror](https://www.npmjs.com/package/codemirror)  [mermaid](https://www.npmjs.com/package/mermaid)

🚀 **Related Components**:   [addon/fold-code](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-code.ts)  [addon/fold](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold.ts)

🚢 **Provides**:

* [**MermaidRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-code-with-mermaid.ts#L29)   Type: CodeRenderer

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-code"

This module provides `MermaidRenderer` for FoldCode addon
so that you can render flowchart / diagram with powerful [mermaid](https://mermaidjs.github.io/)

By default the renderer is enabled. You may disable it by setting `hmdFoldCode.mermaid` to `false`

**Example**: https://laobubu.net/HyperMD/docs/examples/mermaid.html

:hint: to change mermaid configuration

:warning: **Please include mermaid via HTML tag**

mermaid's module declaration is buggy (v8.0.0). HyperMD gave up importing it.

If using RequireJS or bundler (eg. webpack), include `<script src="path/to/mermaid.min.js"></script>` manually,
before RequireJS or `dist/your_app.js`



## fold-emoji-with-emojione

📦 **Third-Party Libs**:   [emojione](https://www.npmjs.com/package/emojione)

🚀 **Related Components**:   [addon/fold-emoji](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts)

🚢 **Provides**:

* [**emojioneChecker**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-emoji-with-emojione.ts#L24)   Type: EmojiChecker
* [**emojioneRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-emoji-with-emojione.ts#L25)   Type: EmojiRenderer

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-emoji"

Use EmojiOne to lookup emojis and render emoji `:smile:` :smile:

:warning: **License**:

Please follow https://www.emojione.com/licenses/free if you use this powerpack.
使用前请注意阅读 EmojiOne 使用许可




## fold-emoji-with-twemoji

📦 **Third-Party Libs**:   [twemoji](https://www.npmjs.com/package/twemoji)

🚀 **Related Components**:   [addon/fold-emoji](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-emoji.ts)

🚢 **Provides**:

* [**setOptions**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-emoji-with-twemoji.ts#L23)   Type: `function(options?: object | Function)`
* [**twemojiChecker**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-emoji-with-twemoji.ts#L27)   Type: EmojiChecker
* [**twemojiRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-emoji-with-twemoji.ts#L29)   Type: EmojiRenderer

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-emoji"

Use [twemoji](https://github.com/twitter/twemoji) to render emoji `:smile:` :smile:

:warning: **License**:

Please follow https://github.com/twitter/twemoji#license if you use this powerpack.
使用前请注意阅读 twemoji 使用许可




## fold-math-with-katex

📦 **Third-Party Libs**:   [katex](https://www.npmjs.com/package/katex)

🚀 **Related Components**:   [addon/fold-math](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts)

🚢 **Provides**:

* [**KatexRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-math-with-katex.ts#L18)

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-math"

Use KaTeX to render TeX formulars.

:warning: In plain browser env, don't forget to load `katex/dist/katex.min.css`.




## fold-math-with-mathjax

📦 **Third-Party Libs**:   [mathjax](https://www.npmjs.com/package/mathjax)

🚀 **Related Components**:   [addon/fold-math](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/fold-math.ts)

🚢 **Provides**:

* [**MathJaxRenderer**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/fold-math-with-mathjax.ts#L29)

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/fold-math"

Use MathJax to render TeX formulars.

:warning: **Configuration Required**

Before loading this module, make sure that you've finished [configuring MathJax](http://docs.mathjax.org/en/latest/configuration.html) like this:

```html
<script type="text/x-mathjax-config">
MathJax.Hub.Config({
    jax: ["input/TeX", "output/HTML-CSS","output/NativeMML","output/SVG"],
    extensions: ["MathMenu.js","MathZoom.js", "AssistiveMML.js", "a11y/accessibility-menu.js"],
    TeX: {
        extensions: ["AMSmath.js","AMSsymbols.js","noErrors.js","noUndefined.js"]
    }
});
</script>
```



## hover-with-marked

📦 **Third-Party Libs**:   [marked](https://www.npmjs.com/package/marked)

🚀 **Related Components**:   [addon/hover](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/hover.ts)

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/hover"

Render tooltip Markdown to HTML, with marked




## insert-file-with-smms

📦 **Third-Party Libs**: (None)

🚀 **Related Components**:   [addon/insert-file](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/insert-file.ts)

🚢 **Provides**:

* [**Upload_One**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/insert-file-with-smms.ts#L25)   Type: `function(file: File, callback: (url?: string) => void)`
* [**SmMsFileHandler**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/insert-file-with-smms.ts#L60)   Type: FileHandler

📕 **Description**:

HyperMD PowerPack "insert-file-with-smms"
This file is distributed under a WTFPL license http://www.wtfpl.net/

POWERPACK for "addon/paste"

Use http://sm.ms as the default destination when user wants to upload images.

:bulb: **Hint**:

Feel free to copy codes from this module, alter `Upload_One`,
and make a new `FileHandler` which uploads images to somewhere else.

:warning: **Attribution Required**

If you are using this PowerPack, please add "Uploaded images are hosted by https://sm.ms" to your app
如果使用了此 PowerPack，请别忘记添加 “由 https://sm.ms 提供图床服务”



## paste-with-turndown

📦 **Third-Party Libs**:   [turndown](https://www.npmjs.com/package/turndown)

🚀 **Related Components**:   [addon/paste](https://github.com/laobubu/HyperMD/tree/a6530e01/src/addon/paste.ts)

🚢 **Provides**:

* [**TurndownConvertor**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/paste-with-turndown.ts#L19)   Type: PasteConvertor
* [**getTurndownService**](https://github.com/laobubu/HyperMD/tree/a6530e01/src/powerpack/paste-with-turndown.ts#L36)

📕 **Description**:

HyperMD, copyright (c) by laobubu
Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE

POWERPACK for "addon/paste"

This module provides `TurndownConvertor`, which can convert copied HTML into Markdown.

:bulb: **Hint**:

This module also supports GFM extensions (eg. tables) via [turndown-plugin-gfm](https://www.npmjs.com/package/turndown-plugin-gfm).
Load `turndown-plugin-gfm` after this module and it will be automatically used while pasting.



